<?php

// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}

// Ensure the trait is loaded
require_once plugin_dir_path(__FILE__) . 'trait-logger.php';


/**
 * Class responsible for interacting with the Altomatic API
 */
class Altomatic_API {
	use LoggerTrait;


    /**
     * API endpoint
     */
    private $api_endpoint = 'https://api.altomatic.ai';

    /**
     * API key
     */
    private $api_key;

    /**
     * Maximum retries for API calls
     */
    private $max_retries = 2;

    /**
     * Initialize the class
     */
    public function __construct() {
        $this->api_key = get_option('altomatic_api_key');

        if (empty($this->api_key)) {
            self::log('No API key configured');
            add_action('admin_notices', function() {
                echo '<div class="notice notice-error"><p>' .
                     esc_html__('Altomatic: API key is not configured. Please enter your API key in the settings.', 'altomatic') .
                     '</p></div>';
            });
        } else {
            self::log('Using API key: ' . substr($this->api_key, 0, 8) . '...');
        }
    }

    /**
     * Map file extension to API format
     * @param mixed $format
     */
    private function map_format($format) {
        $format = strtolower($format);
        switch ($format) {
            case 'jpg':
                return 'jpeg';
            case 'jpeg':
                return 'jpeg';
            default:
                return $format;
        }
    }

    /**
     * Map API format to file extension
     * @param mixed $format
     */
    private function map_extension($format) {
        $format = strtolower($format);
        switch ($format) {
            case 'jpeg':
                return 'jpg';
            default:
                return $format;
        }
    }

    /**
     * Upload an image to get optimization URLs
     * @param mixed $file_path
     */
    public function upload_image($file_path) {
        $this->log("Starting image optimization for file: $file_path");

        // Get file info for logging
        $file_size = filesize($file_path);
        $image_info = getimagesize($file_path);
        $mime_type = $image_info['mime'];
        $width = $image_info[0];
        $height = $image_info[1];

        $this->log("File size: " . round($file_size / 1024) . " KB");
        $this->log("MIME type: $mime_type");
        $this->log("Image dimensions: {$width}x{$height}");

        // Try binary upload first
        $this->log("Upload URL: " . $this->get_api_url('v1/image'));

        $response = wp_remote_post($this->get_api_url('v1/image'), array(
            'headers' => array(
                'Authorization' => 'Bearer ' . $this->api_key,
                'Content-Type' => $mime_type
            ),
            'body' => file_get_contents($file_path),
            'timeout' => 30,
            'sslverify' => false
        ));

        if (is_wp_error($response)) {
            return $response;
        }

        $code = wp_remote_retrieve_response_code($response);
        if ($code !== 200) {
            return new WP_Error(
                'api_error',
                'Failed to upload image. Status code: ' . $code
            );
        }

        $body = json_decode(wp_remote_retrieve_body($response), true);
        if (!is_array($body) || !isset($body['links'])) {
            return new WP_Error(
                'invalid_response',
                'Invalid response from API'
            );
        }

        // Map the format in links if needed
        $format = strtolower(pathinfo($file_path, PATHINFO_EXTENSION));
        $api_format = $this->map_format($format);

        if ($format !== $api_format && isset($body['links'][$api_format])) {
            // If we have a jpg file but API returns jpeg format, add it as both
            $body['links'][$format] = $body['links'][$api_format];
        }

        return $body;
    }

    /**
     * Get optimized image URL with optional size parameters
     * @param mixed $image_id
     * @param mixed $format
     * @param mixed $width
     * @param mixed $height
     */
    private function get_optimized_url($image_id, $format = 'webp', $width = null, $height = null) {
        $url = $this->get_api_url("v1/image/{$image_id}/optimized/{$format}");

        $params = array();
        if ($width) {
            $params['width'] = $width;
            $this->log("Setting width to {$width}px");
        }
        if ($height) {
            $params['height'] = $height;
            $this->log("Setting height to {$height}px");
        }

        if (!empty($params)) {
            $url .= '?' . http_build_query($params);
            $this->log("Adding size parameters to URL: " . http_build_query($params));
        }

        $this->log("Generated optimized URL: {$url}");
        return $url;
    }
    /**
     * @param mixed $image_id
     */
    public function get_regenerated_alt_text_link($image_id) {
        return $this->get_api_url("v1/image/{$image_id}/alt-text?regenerate=true");
    }


    /**
     * Validate API key
     * @param mixed $api_key
     */
    public function validate_key($api_key = null) {
        $key_to_validate = $api_key ?? $this->api_key;

        if (empty($key_to_validate)) {
            $this->log('❌ API: No API key provided for validation');
            return new WP_Error('no_api_key', __('No API key provided', 'altomatic'));
        }

        // Use stats endpoint for validation
        $url = $this->get_api_url('v1/stats');
        self::log("Validating key at: {$url}");
        self::log("Using key: " . substr($key_to_validate, 0, 8) . '...');

        $response = wp_remote_get($url, array(
            'headers' => array(
                'Authorization' => 'Bearer ' . $key_to_validate,
                'Accept' => 'application/json'
            ),
            'timeout' => 15,
            'sslverify' => false,
            'user-agent' => 'WordPress/' . get_bloginfo('version') . '; ' . get_bloginfo('url')
        ));

        if (is_wp_error($response)) {
            $this->log('❌ API: Validation request failed - ' . $response->get_error_message());
            return $response;
        }

        $status_code = wp_remote_retrieve_response_code($response);
        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body, true);

        self::log("Validation response - Status: {$status_code}");
        self::log("Validation body: {$body}");

        if ($status_code === 200 && !empty($data)) {
            $this->log('✅ API: Key validation successful');
            return array(
                'valid' => true,
                'message' => __('API key is valid', 'altomatic'),
                'data' => array(
                    'credits' => $data['credits'],
                    'optimized_images' => $data['optimized_images'],
                    'alt_text_generations' => $data['alt_text_generations']
                )
            );
        }

        $this->log('❌ API: Key validation failed - Status: ' . $status_code);

        return new WP_Error(
            'invalid_api_key',
            __('Invalid API key', 'altomatic'),
            array('status' => $status_code)
        );
    }

    /**
     * Check if the system can use binary upload method
     *
     * @return bool True if binary upload is possible
     */
    private function can_use_binary_upload() {
        // Check for required PHP functions
        if (!function_exists('curl_init') ||
            !function_exists('curl_setopt') ||
            !function_exists('curl_exec') ||
            !function_exists('curl_close')) {
            self::log("CURL functions not available for binary upload");
            return false;
        }

        // Check if we can read files into memory
        if (!function_exists('file_get_contents')) {
            self::log("file_get_contents function not available for binary upload");
            return false;
        }

        // Check if we can get mime type
        if (!function_exists('mime_content_type')) {
            self::log("mime_content_type function not available for binary upload");
            return false;
        }

        self::log("System can use binary upload method");
        return true;
    }

    /**
     * Upload an image using binary data
     *
     * @param string $url The API URL
     * @param string $file_path Path to the file to upload
     * @return array|WP_Error
     */
    private function upload_binary($url, $file_path) {
        self::log("Binary upload URL: {$url}");

        // Make sure CURL is available - double check even though we checked in can_use_binary_upload
        if (!function_exists('curl_init')) {
            self::log("CURL not available for binary upload");
            return new WP_Error('curl_not_available', __('CURL not available', 'altomatic'));
        }

        // Get MIME type
        $mime_type = mime_content_type($file_path);
        if (!$mime_type || strpos($mime_type, 'image/') !== 0) {
            self::log("Invalid MIME type for binary upload: {$mime_type}");
            return new WP_Error('invalid_mime_type', __('Invalid MIME type', 'altomatic'));
        }

        // Read file content
        $file_content = file_get_contents($file_path);
        if (!$file_content) {
            self::log("Could not read file: {$file_path}");
            return new WP_Error('file_read_error', __('Could not read file', 'altomatic'));
        }

        // Create multipart boundary
        $boundary = wp_generate_password(24, false);

        // Build multipart data
        $data = '';
        $data .= "--{$boundary}\r\n";
        $data .= "Content-Disposition: form-data; name=\"image\"; filename=\"" . basename($file_path) . "\"\r\n";
        $data .= "Content-Type: {$mime_type}\r\n\r\n";
        $data .= $file_content . "\r\n";
        $data .= "--{$boundary}--\r\n";

        $retry_count = 0;
        $max_retries = 5; // Increased from 2 to 5 for capacity errors
        $base_delay = 5; // Start with 5 second delay

        while ($retry_count <= $max_retries) {
            // Initialize CURL
            $ch = curl_init($url);

            // Set CURL options
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Content-Type: multipart/form-data; boundary=' . $boundary,
                'Authorization: Bearer ' . $this->api_key,
                'Accept: application/json',
                'User-Agent: WordPress Plugin/1.0'
            ));
            curl_setopt($ch, CURLOPT_TIMEOUT, 120); // Increased timeout for larger images
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);

            // Execute the request
            $response_body = curl_exec($ch);
            $response_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            $curl_error = curl_error($ch);

            curl_close($ch);

            // Check for CURL errors
            if ($response_body === false) {
                self::log("CURL error on attempt #{$retry_count}: {$curl_error}");

                if ($retry_count < $max_retries) {
                    $retry_count++;
                    $delay = $base_delay * pow(2, $retry_count - 1); // Exponential backoff
                    self::log("Retrying in {$delay} seconds (attempt {$retry_count} of {$max_retries})");
                    sleep($delay);
                    continue;
                }
                return new WP_Error('curl_error', $curl_error);
            }

            // Parse response for specific error cases
            $response_data = json_decode($response_body, true);
            self::log("Response body: " . $response_body);

            if ($response_code === 500) {
                if (isset($response_data['error'])) {
                    self::log("Server error: " . $response_data['error']);
                    return new WP_Error('api_error', $response_data['error']);
                }
            }

            // If we got a 200 response, we're done
            if ($response_code === 200) {
                self::log("Binary upload successful on attempt #{$retry_count}");

                // Parse the successful response
                $data = json_decode($response_body, true);
                if (json_last_error() !== JSON_ERROR_NONE) {
                    self::log("JSON decode error: " . json_last_error_msg());
                    return new WP_Error('json_decode_error', __('Invalid JSON response', 'altomatic'));
                }

                return $data;
            }

            // If we get here, we got an unexpected response code
            self::log("Unexpected response code {$response_code} on attempt #{$retry_count}");

            if ($retry_count < $max_retries) {
                $retry_count++;
                $delay = $base_delay * pow(2, $retry_count - 1); // Exponential backoff
                self::log("Retrying in {$delay} seconds (attempt {$retry_count} of {$max_retries})");
                sleep($delay);
                continue;
            }

            return new WP_Error(
                'api_error',
                sprintf(
                    /* translators: %s: API response code */
                    __('API error: %s', 'altomatic'),
                    $response_code
                )
            );
        }

        // If we get here, we've exhausted all retries
        self::log("Exhausted all retries");
        return new WP_Error('max_retries_exceeded', __('Maximum retry attempts exceeded', 'altomatic'));
    }

    /**
     * Upload image using CURL for better control
     * @param mixed $file_path
     * @param mixed $url
     * @param mixed $query_args
     */
    private function upload_with_curl($file_path, $url, $query_args) {
        // Add query parameters to URL
        $url = add_query_arg($query_args, $url);
        self::log("CURL URL with parameters: {$url}");

        // Make sure CURL is available
        if (!function_exists('curl_init')) {
            self::log("CURL not available");
            return new WP_Error('curl_not_available', __('CURL not available', 'altomatic'));
        }

        // Initialize CURL
        $ch = curl_init();

        // Create a CURLFile object
        $cfile = curl_file_create(
            $file_path,
            mime_content_type($file_path),
            basename($file_path)
        );

        // Set CURL options
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, array('image' => $cfile));
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Authorization: Bearer ' . $this->api_key,
            'Accept: application/json'
        ));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 90);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        // Execute the request
        $response_body = curl_exec($ch);
        $response_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $curl_error = curl_error($ch);

        // Close CURL session
        curl_close($ch);

        // Log response
        self::log("CURL Response code: {$response_code}");

        if (!empty($curl_error)) {
            self::log("CURL Error: {$curl_error}");
            return new WP_Error('curl_error', $curl_error);
        }

        if ($response_code !== 200) {
            self::log("CURL Bad response code: {$response_code}");
            self::log("CURL Response body: {$response_body}");

            return new WP_Error(
                'api_error',
                sprintf(
                    /* translators: %s: API response code */
                    __('API error: %s', 'altomatic'), 
                    $response_code
                    )
            );
        }

        // Parse response
        if (empty($response_body)) {
            self::log("CURL Empty response body");
            return new WP_Error('empty_response', __('Empty response from API', 'altomatic'));
        }

        $data = json_decode($response_body, true);
        if (json_last_error() !== JSON_ERROR_NONE) {
            self::log("CURL JSON decode error: " . json_last_error_msg());
            return new WP_Error('json_decode_error', __('Invalid JSON response', 'altomatic'));
        }

        self::log("CURL Received valid response");

        return $data;
    }

    /**
     * Upload image using WordPress HTTP API
     * @param mixed $file_path
     * @param mixed $url
     * @param mixed $query_args
     */
    private function upload_with_wp_http($file_path, $url, $query_args) {
        // Add query parameters to URL
        $url = add_query_arg($query_args, $url);
        self::log("WP HTTP URL with parameters: {$url}");

        // Create multipart/form-data boundary
        $boundary = wp_generate_password(24, false);

        // Read file content
        $file_content = file_get_contents($file_path);
        if (!$file_content) {
            self::log("Could not read file: {$file_path}");
            return new WP_Error('file_read_error', __('Could not read file', 'altomatic'));
        }

        // Build multipart body
        $body = "--{$boundary}\r\n";
        $body .= "Content-Disposition: form-data; name=\"image\"; filename=\"" . basename($file_path) . "\"\r\n";
        $body .= "Content-Type: " . mime_content_type($file_path) . "\r\n\r\n";
        $body .= $file_content . "\r\n";
        $body .= "--{$boundary}--\r\n";

        // Make the API request
        $response = wp_remote_post(
            $url,
            array(
                'headers' => array(
                    'Authorization' => 'Bearer ' . $this->api_key,
                    'Content-Type' => 'multipart/form-data; boundary=' . $boundary,
                    'Accept' => 'application/json'
                ),
                'body' => $body,
                'timeout' => 90,
                'sslverify' => false
            )
        );

        // Check for errors
        if (is_wp_error($response)) {
            self::log("Request error: " . $response->get_error_message());
            return $response;
        }

        // Check response code
        $response_code = wp_remote_retrieve_response_code($response);
        if ($response_code !== 200) {
            self::log("Bad response code: {$response_code}");

            return new WP_Error(
                'api_error',
                
                sprintf(
                    /* translators: %s: API response code */
                    __('API error: %s', 'altomatic'),
                    $response_code
                )
            );
        }

        // Get response body
        $body = wp_remote_retrieve_body($response);
        if (empty($body)) {
            self::log("Empty response body");
            return new WP_Error('empty_response', __('Empty response from API', 'altomatic'));
        }

        // Decode JSON
        $data = json_decode($body, true);
        if (json_last_error() !== JSON_ERROR_NONE) {
            self::log("JSON decode error: " . json_last_error_msg());
            return new WP_Error('json_decode_error', __('Invalid JSON response', 'altomatic'));
        }

        self::log("Received valid response");

        return $data;
    }

    /**
     * Parse API response to standardized format
     * @param mixed $data
     */
    private function parse_api_response($data) {
        $result = array();

        // Handle the new API response format
        if (isset($data['optimized_images'])) {
            self::log("Using new API response format");

            // Get the MIME type to determine original format
            $result['url'] = null;
            $result['webp_url'] = null;

            // Different image formats available
            foreach ($data['optimized_images'] as $format => $url) {
                self::log("Got optimized URL for format {$format}: {$url}");

                // Set the main URL to the first format, will be overridden if we find a better match
                if ($result['url'] === null) {
                    $result['url'] = $url;
                }

                // Always capture WebP URL
                if ($format === 'webp') {
                    $result['webp_url'] = $url;
                }

                // Always capture JPEG URL
                if ($format === 'jpeg') {
                    $result['jpeg_url'] = $url;
                }

                // Always capture PNG URL
                if ($format === 'png') {
                    $result['png_url'] = $url;
                }
            }

            // Include alt text if available
            if (isset($data['alt_text'])) {
                $result['alt_text'] = $data['alt_text'];
                self::log("Got alt text: " . $result['alt_text']);
            }

        } else if (isset($data['url'])) {
            self::log("Using legacy API response format");

            // Legacy API format
            $result = array(
                'url' => $data['url'],
                'webp_url' => isset($data['webp_url']) ? $data['webp_url'] : null,
                'alt_text' => isset($data['alt_text']) ? $data['alt_text'] : null,
            );

            self::log("Got optimized URL: " . $result['url']);

            if ($result['webp_url']) {
                self::log("Got WebP URL: " . $result['webp_url']);
            }
        } else {
            self::log("Unexpected response format");
            return new WP_Error('unexpected_format', __('Unexpected response format', 'altomatic'));
        }

        // Validate we have a URL
        if (empty($result['url'])) {
            self::log("No URL in response");
            return new WP_Error('no_url', __('No URL in API response', 'altomatic'));
        }

        self::log("Optimization completed successfully");
        return $result;
    }

    /**
     * Get account usage and limits
     */
    public function get_usage() {
        $response = wp_remote_get($this->api_endpoint . '/account/usage', array(
            'headers' => array(
                'Authorization' => 'Bearer ' . $this->api_key
            ),
            'timeout' => 15
        ));

        if (is_wp_error($response)) {
            return $response;
        }

        $body = json_decode(wp_remote_retrieve_body($response), true);
        $code = wp_remote_retrieve_response_code($response);

        if ($code !== 200) {
            return new WP_Error(
                'api_error',
                isset($body['message']) ? $body['message'] : __('API error', 'altomatic')
            );
        }

        return $body;
    }

    /**
     * Constructs a full API URL from a given endpoint
     * @param mixed $endpoint
     */
    private function get_api_url($endpoint) {
        // Make sure to add v1 prefix if not already present
        if (strpos($endpoint, 'v1/') !== 0) {
            $endpoint = 'v1/' . $endpoint;
        }
        return $this->api_endpoint . '/' . ltrim($endpoint, '/');
    }

    /**
     * Download and save an image from a URL
     * @param mixed $url
     * @param mixed $file_path
     * @param mixed $query_params
     */
    public function download_and_save_image($url, $file_path, $query_params = array()) {
        if (!empty($query_params)) {
            $url = add_query_arg($query_params, $url);
        }

        self::log("Downloading image from {$url} to {$file_path}");

        // Initialize CURL
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Authorization: Bearer ' . $this->api_key,
            'Accept: image/*'
        ));

        // Execute request
        $content = curl_exec($ch);
        $status_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if ($status_code !== 200) {
            self::log("Download failed - HTTP {$status_code} - {$url}: {$content}");
            return new WP_Error('download_failed', "HTTP error {$status_code}");
        }

        if (empty($content)) {
            self::log("Empty response body");
            return new WP_Error('empty_response', 'Empty response body');
        }

        // Create directory if it doesn't exist
        $dir = dirname($file_path);
        if (!file_exists($dir)) {
            wp_mkdir_p($dir);
        }

        // Save the file
        $result = file_put_contents($file_path, $content);
        if ($result === false) {
            self::log("Failed to save file");
            return new WP_Error('save_failed', 'Failed to save file');
        }

        self::log("Successfully downloaded and saved image");
        return true;
    }

    /**
     * Get alt text for an image using the provided URL
     * @param mixed $url
     */
    public function get_alt_text($url) {
        $response = wp_remote_get($url, array(
            'headers' => array(
                'Authorization' => 'Bearer ' . $this->api_key,
                'Accept' => 'application/json'
            ),
            'timeout' => 30,
            'sslverify' => false
        ));

        if (is_wp_error($response)) {
            return $response;
        }

        $code = wp_remote_retrieve_response_code($response);
        if ($code !== 200) {
            return new WP_Error(
                'api_error',
                'Failed to get alt text. Status code: ' . $code
            );
        }

        $body = json_decode(wp_remote_retrieve_body($response), true);
        if (!is_array($body) || !isset($body['alt_text'])) {
            return new WP_Error(
                'invalid_response',
                'Invalid response from alt text API'
            );
        }

        return $body;
    }

    /**
     * Get credit information
     */
    public function get_credit_info() {
        // We can just reuse validate_key since it now returns the stats data
        $result = $this->validate_key();

        if (is_wp_error($result)) {
            return $result;
        }

        return array(
            'success' => true,
            'data' => array(
                'remaining_credits' => $result['data']['credits']['remaining'],
                'total_credits' => $result['data']['credits']['total'],
                'used_credits' => $result['data']['credits']['used'],
                'monthly_optimizations' => $result['data']['optimized_images']['monthly'],
                'monthly_alt_text' => $result['data']['alt_text_generations']['monthly']
            )
        );
    }
}
